@@ -203,19 +203,17 @@ def check_filter(response, comp = 0, filter_number = 0):
     return response.json()[0]['attachment']['body'][comp]['details']['filters'][filter_number]['field'], response.json()[0]['attachment']['body'][comp]['details']['filters'][filter_number]['operator'], response.json()[0]['attachment']['body'][comp]['details']['filters'][filter_number]['values']
 
 
-
 #check if a button component has a specific label and link
 def check_button(response, comp = 0):
     
     return response.json()[0]['attachment']['body'][comp]['details']['label'], response.json()[0]['attachment']['body'][comp]['details']['link']
 
 
-
 ## run the query test
-
 def run_test(query = '',
              n_cards = None, check_component_types = None, 
-             check_table_columns = None, check_filters_existance = None, check_button_existace = None,
+             check_table_columns = None, check_filters_existance = None,
+             check_button_existace = None,
            
             print_response = False):
     
@@ -238,79 +236,159 @@ def run_test(query = '',
     response = run_query(glob_token, agent_id, query)
     
     
-    ##check all the test for a query
-    try:
-        assert response.status_code == 200
+    failed_message = ""
+    
+    failed = False
+    
+    
+    #failed = dict()
+    
+    #check the status response
+    if response.status_code == 200:
         
         print('Response 200')
         
-        if n_cards:
-        
-            assert num_of_cards(response) == n_cards
+        if 'text' in response.json()[0]['attachment']['body'][0]['details'] and response.json()[0]['attachment']['body'][0]['details']['text'] == 'Could you be more specific?':
             
-            print('Number of cards expected passed')
-        
-        
-        if check_component_types:
+            print('Card <Could you be more specific?>')
             
-            for position, component in enumerate(check_component_types):
-                
-                assert check_component(response, position) == component
-                
-            print('Check for Components type passed')
+            failed = True
             
-                
-        if check_table_columns:
+            failed_message += "<br><b>Card returned:</b> Could you be more specific?\n"
             
-            for position, columns in enumerate(check_table_columns):
+        else:
+            #check the number of cards
+            if n_cards:
+
+                if num_of_cards(response) == n_cards:
+                    print('\n')
+                    print('Number of cards expected passed')
+
+
+                else:
+
+                    failed = True
+
+                    #failed['Number or cards'] = num_of_cards(response)
+
+                    failed_message += "<br><b>Expected number of cards</b>: {}, but given {}".format(n_cards, num_of_cards(response))
+                        
                 
-                if columns != []:
                 
-                    assert check_col_names(response, position) == set(columns)               
-            
-            print('Check for column names in table passed')
+            #check comp types
+    
+            if check_component_types:
+
+                print('\n')
+
+                for position, component in enumerate(check_component_types):
+
+                    if check_component(response, position) == component:
+
+                        print('Check for component {} passed'.format(component))
+
+                    else:
+
+                        failed = True
+
+                        print('Check for Components failed. Expected {}, but given {}'.format(component, check_component(response, position)))
+
+
+                        failed_message += "<br><b>Expected component in position {}</b>: \"{}\", but given \"{}\".".format(position, component, check_component(response, position))
             
     
-        if check_filters_existance:
-            
-            for position, filters in enumerate(check_filters_existance):
-                
-                if filters != []:
+            #check tab cols
+    
+            if check_table_columns:
+
+                print('\n')
+
+                for position, columns in enumerate(check_table_columns):
+
+                    if columns != []:  
+
+                        if check_col_names(response, position) == set(columns):
+
+                            print('Check for column names in table passed')
+
+
+                        else:
+
+                            failed = True
+
+                            print('Check for column names in table failed. Expected {} but given {}'.format(columns, list(check_col_names(response, position))))
+
+
+                            failed_message += "<br><b>Expected columns</b>: <pre>{}</pre>".format(pprint.pformat(columns))
+                                
                     
-                    for filter_number, f in enumerate(filters):
-                        
-                        
-                    
-                        assert check_filter(response, position, filter_number) == f
-                
-            print('Check for filters passed')
-            
         
-        if check_button_existace:
+            if check_filters_existance:
             
-            for position, button in enumerate(check_button_existace):
-                
-                if button != []:
-                
-                    assert list(check_button(response, position)) == button
-            
-            print('Check for button passed')
-            
-            
-        message_body += "<br><b style='color:green;'>Test Successful</b> for query: {}".format(query)
+                for position, filters in enumerate(check_filters_existance):
+
+                    if filters != []:
+
+                        for filter_number, f in enumerate(filters):
+
+                            if check_filter(response, position, filter_number) == f:
+
+                                print('Check for filters {} passed'.format(f))
+
+                            else:
+
+                                failed = True
+
+                                print('Check for filters {} failed, given {}'.format(f, check_filter(response, position, filter_number)))
+
+                                failed_message += "<br><b>Expected filters</b>: <pre>{}</pre> but given <pre>{}</pre>".format(pprint.pformat(f), pprint.pformat(check_filter(response, position, filter_number)))
         
+        
+            if check_button_existace:
+            
+                for position, button in enumerate(check_button_existace):
+
+                    if button != []:
+
+                        if list(check_button(response, position)) == button:
+
+                            print('Check for button passed')
+
+                        else:
+
+                            failed = True
+
+                            print('Check for button failed. Expected {}, but given {}'.format(button, list(check_button(response, position))))
+
+                            failed_message += "<br><b>Expected button</b>: <pre>{}</pre> but given <pre>{}</pre>".format(pprint.pformat(button), pprint.pformat(list(check_button(response, position))))
+    
+    
+    else:
+    
+    
+        failed = True
+        
+        status = response.status_code
+        
+        print('Response', status)
+        
+        #failed['Status Response'] = status
+        
+        failed_message += "<br><b>Response status</b> {}".format(status)
+    
+    
+    
+    if not failed:
+    
         test_passed += 1
         
         total_passed += 1
         
         print('All Check Passed\n')
         
+        message_body += "<br><b style='color:green;'>Test Successful</b> for query: <i>{}</i>".format(query)
         
-    except AssertionError:
-    
-        status = response.status_code
-    
-        message_body += "<br><b style='color:red;'>Test Failed</b> Reponse status {} for query: {}".format(status, query)
+    else:
     
         test_failed += 1
         
@@ -318,10 +396,10 @@ def run_test(query = '',
         
         print('Test Failed\n')
         
-    
-    #message_body += "<br><b>Response:\n</b>" + '<pre>' + pprint.pformat(response.json()[0]) + '\n</pre>'
-    
-    #message_body += "<details><summary><br><b>Response:\n</b></summary>" + '<pre>' + pprint.pformat(response.json()[0]) + '\n</pre></details>'
+        message_body += "<br><b style='color:red;'>Test Failed</b> for query: <i>{}</i>".format(query)
+        
+        message_body += failed_message
+        
     
     try:
         card_0 = response.json()[0]['attachment']['body']
@@ -330,13 +408,14 @@ def run_test(query = '',
 
             if card_0[i]['component_type'] == 'table':
 
-                message_body += "<br><b>Table Row 1:\n</b><pre>" + pprint.pformat(card_0[i]['details']['columns']) + '\n</pre>'
+                message_body += "<br><b>Data Response (Row 1):\n</b><pre>" + pprint.pformat(card_0[i]['details']['columns']) + '\n</pre>'
                 message_body += "<pre>" + pprint.pformat(card_0[i]['details']['rows'][0]) + '\n</pre>'
 
     except:
         
         pass
 
+    
     if print_response:
         
     #this print is helpful for debugging code
@@ -354,7 +433,6 @@ def run_test(query = '',
     
     return message_body
 
-
 ## finalize agent test message body after running one o more run_test
 
 def finalize_agent_test():
